#If you are looking into the sourcecode, sorry about it being messy... This was actually my first script I started when learning python (it's taken a while to finish), so learning as I go and my laziness have made for some sloppy code...

import bpy
import random
import glob
import os
import sys
from bpy_extras.image_utils import load_image
from bpy_extras.io_utils import ImportHelper
from bpy.app.handlers import persistent


bl_info = {
    "name": "Snu Slideshow Generator",
    "description": "Assists in creating image slideshows with a variety of options.",
    "author": "Hudson Barkley (Snu)",
    "version": (0, 7, 9),
    "blender": (2, 76, 0),
    "location": "Properties Window, Scene Tab",
    "wiki_url": "none yet",
    "category": "Import-Export"
}


#This is a list of transform dictionary objects
transforms = [
    {
        'name':'None',
        'influence': [(0, 1)],
        'zLoc': [(0, 1)],
        'zRot': [(0, 1)],
        'xLoc': [(0, 1)]
    },
    {
        'name':'Move Left Complete With Pause',
        'xLoc': [(-1.556, 0), (0, 1), (1.556, 0)]
    },
    {
        'name':'Move Right Complete With Pause',
        'xLoc': [(1.556, 0), (0, 1), (-1.556, 0)]
    },
    {
        'name':'Move Left Little',
        'xLoc': [(-0.2, 1), (0.2, 1)]
    },
    {
        'name':'Move Right Little',
        'xLoc': [(0.2, 1), (-0.2, 1)]
    },
    {
        'name':'Rotate Left Around Top',
        'zLoc': [(-0.3, 1)],
        'zRot': [(-10, 1), (10, 1)],
        'xLoc': [(-0.125, 1), (0.125, 1)]
    },
    {
        'name':'Rotate Left Around Bottom',
        'zLoc': [(-0.3, 1)],
        'zRot': [(10, 1), (-10, 1)],
        'xLoc': [(-0.125, 1), (0.125, 1)]
    },
    {
        'name':'Rotate Right Around Top',
        'zLoc': [(-0.3, 1)],
        'zRot': [(10, 1), (-10, 1)],
        'xLoc': [(0.125, 1), (-0.125, 1)]
    },
    {
        'name':'Rotate Right Around Bottom',
        'zLoc': [(-0.3, 1)],
        'zRot': [(-10, 1), (10, 1)],
        'xLoc': [(0.125, 1), (-0.125, 1)]
    },
    {
        'name':'Pan To Target',
        'influence': [(0, 1), (1, 1)]
    },
    {
        'name':'Rotate Left',
        'zLoc': [(-0.3, 1)],
        'zRot': [(10, 1), (-10, 1)]
    },
    {
        'name':'Rotate Right',
        'zLoc': [(-0.3, 1)],
        'zRot': [(-10, 1), (10, 1)]
    },
    {
        'name':'Zoom In',
        'influence': [(0, 1), (1, 1)],
        'zLoc': [(0, 1), (-0.3, 1)]
    },
    {
        'name':'Zoom Out',
        'influence': [(1, 1), (0, 1)],
        'zLoc': [(-0.3, 1), (0, 1)]
    },
    {
        'name':'Zoom In And Rotate Left',
        'influence': [(0, 1), (1, 1)],
        'zLoc': [(0, 1), (-0.3, 1)],
        'zRot': [(0, 1), (-10, 1)]
    },
    {
        'name':'Zoom In And Rotate Right',
        'influence': [(0, 1), (1, 1)],
        'zLoc': [(0, 1), (-0.3, 1)],
        'zRot': [(0, 1), (10, 1)]
    },
    {
        'name':'Zoom Out And Rotate Left',
        'influence': [(1, 1), (0, 1)],
        'zLoc': [(-0.3, 1), (0, 1)],
        'zRot': [(10, 1), (0, 1)]
    },
    {
        'name':'Zoom Out And Rotate Right',
        'influence': [(1, 1), (0, 1)],
        'zLoc': [(-0.3, 1), (0, 1)],
        'zRot': [(-10, 1), (0, 1)]
    }
]


@persistent
def slideshow_autoupdate(self=None, context=bpy.context):
    #Auto-running modal function to update the slideshow scene so the slides stay ordered properly
    if (bpy.context.scene.name == 'Slideshow Generator'):
        update_order()


def create_scene(oldscene, scenename):
    #Creates a new scene and copies over render settings from current scene
    newscene = bpy.data.scenes.new(scenename)
    bpy.context.screen.scene = newscene
    
    #These loops are needed because there apparently is no way to easily copy the render settings from one scene to another
    for property in oldscene.render.bl_rna.properties:
        #Copy the general render settings
        if not property.is_readonly:
            value = eval('oldscene.render.'+property.identifier)
            setattr(newscene.render, property.identifier, value)
    for property in oldscene.render.image_settings.bl_rna.properties:
        #Copy the image/video encoding settings
        if not property.is_readonly:
            value = eval('oldscene.render.image_settings.'+property.identifier)
            setattr(newscene.render.image_settings, property.identifier, value)
    for property in oldscene.render.ffmpeg.bl_rna.properties:
        #Copy the video encoder settings
        if not property.is_readonly:
            value = eval('oldscene.render.ffmpeg.'+property.identifier)
            setattr(newscene.render.ffmpeg, property.identifier, value)

    #Set variables that are specific to the slideshow scene
    newscene.render.engine = 'BLENDER_RENDER'
    newscene.game_settings.material_mode = 'GLSL'
    newscene.layers = [True, True, True, True, True, True, True, True, True, True, True, True, True, True, True, True, True, True, True, True]
    newscene.render.resolution_percentage = 100
    newscene.render.antialiasing_samples = '5'
    newscene.render.pixel_filter_type = 'BOX'
    newscene.render.image_settings.color_mode = 'RGB'
    return newscene


def import_slideshow_image(image, imageNumber, slideshowSlideLength, generatorScene, video=False, lastImage=None):
    #This function will import an image into the generator scene and create the objects needed for modifying slides in the generator
    if (len(image.name) > 20):
        image.name = image.name[0:19]
    if video:
        print('Importing video '+str(imageNumber + 1)+', filename: '+image.name)
    else:
        print('Importing image '+str(imageNumber + 1)+', filename: '+image.name)

    #create and set up image plane
    bpy.context.scene.cursor_location = (0.0, 0.0, 0.0)
    imageMesh = bpy.data.meshes.new(name=image.name)
    ix = ((image.size[0] / image.size[1])/2)
    iy = 0.5
    verts = [(-ix,iy,0.0), (ix,iy,0.0), (ix,-iy,0.0), (-ix,-iy,0.0)]
    faces = [(3, 2, 1, 0)]
    imageMesh.from_pydata(verts, [], faces)
    imagePlane = bpy.data.objects.new(name=image.name, object_data=imageMesh)
    generatorScene.objects.link(imagePlane)
    imagePlane.slideshow.name = imagePlane.name

    add_constraints(imagePlane, 'Plane')
    
    if video:
        imagePlane.slideshow.locklength = True
        imagePlane.slideshow.lockextra = True
        imagePlane.slideshow.locktransform = True
        imagePlane.slideshow.videomaxlength = image.frame_duration
        imagePlane.slideshow.videofile = image.filepath
        slideshowSlideLength = image.frame_duration / generatorScene.render.fps

    #set up and apply material to plane
    imageMaterial = bpy.data.materials.new(imagePlane.name)
    imageMaterial.use_shadeless = True
    imageTexture = bpy.data.textures.new(imagePlane.name, type = 'IMAGE')
    imageTexture.image = image
    imageTextureSlot = imageMaterial.texture_slots.add()
    imageTextureSlot.texture = imageTexture
    imagePlane.data.uv_textures.new()
    imageTextureSlot.texture_coords = 'UV'
    imagePlane.data.materials.append(imageMaterial) #set material to plane
    if video:
        imageTexture.image_user.frame_duration = image.frame_duration
        imageTexture.image_user.use_auto_refresh = True
        imageTexture.image_user.frame_offset = 0

    #set up transform and extras
    if not video:
        randomized = []
        hidden = generatorScene.slideshowTransformsHidden.split(";")
        for transform in transforms:
            if transform['name'] not in hidden:
                if lastImage:
                    if transform['name'] != lastImage.slideshow.transform:
                        randomized.append(transform)
                else:
                    randomized.append(transform)
        if len(randomized) == 0:
            transform = transforms[0]
        else:
            transform = randomized[random.randint(0, (len(randomized) - 1))]

        randomized = []
        hidden = generatorScene.slideshowExtrasHidden.split(";")
        extras = list_extras()
        for extra in extras:
            if extra not in hidden:
                if lastImage:
                    if extra != lastImage.slideshow.extra:
                        randomized.append(extra)
                else:
                    randomized.append(extra)
        if len(randomized) == 0:
            extra = "None"
        else:
            extra = randomized[random.randint(0, (len(randomized) - 1))]
        
        randomized = []
        for extraTexturePreset in generatorScene.slideshowExtraTexturePresets:
            if lastImage:
                if extraTexturePreset.path != lastImage.slideshow.extratexture:
                    randomized.append(extraTexturePreset.path)
            else:
                randomized.append(extraTexturePreset.path)
        if len(randomized) == 0:
            extraTexture = 'None'
        else:
            if len(generatorScene.slideshowExtraTexturePresets) > 1:
                extraTexture = randomized[random.randint(0, (len(randomized) - 1))]
            else:
                extraTexture = generatorScene.slideshowExtraTexturePresets[0].path

        #add target empty
        targetEmpty = bpy.data.objects.new(name=imagePlane.name+' Target', object_data=None)
        targetEmpty.parent = imagePlane
        targetEmpty.empty_draw_size = 1
        add_constraints(targetEmpty, 'Target')
        imagePlane.slideshow.target = targetEmpty.name
        generatorScene.objects.link(targetEmpty)

        #add view empty
        viewEmpty = bpy.data.objects.new(name=imagePlane.name+' View', object_data=None)
        viewEmpty.parent = imagePlane
        viewEmpty.empty_draw_size = 1
        viewEmpty.scale = .888888889, .5, .001
        viewEmpty.empty_draw_type = 'CUBE'
        add_constraints(viewEmpty, 'View')
        imagePlane.slideshow.view = viewEmpty.name
        generatorScene.objects.link(viewEmpty)
    
    #add text next to plane saying which index number it is
    indexTextName = imagePlane.name+' Index'
    indexTextData = bpy.data.curves.new(name=indexTextName, type='FONT')
    indexText = bpy.data.objects.new(name=indexTextName, object_data=indexTextData)
    generatorScene.objects.link(indexText)
    indexText.parent = imagePlane
    indexText.location = ((-1, -.33, 0))
    indexText.data.align_x = 'RIGHT'
    add_constraints(indexText, 'Text')

    if not video:
        #add text next to plane saying which transform was picked
        transformTextName = imagePlane.name+' Transform'
        transformTextData = bpy.data.curves.new(name=transformTextName, type='FONT')
        transformText = bpy.data.objects.new(name=transformTextName, object_data=transformTextData)
        generatorScene.objects.link(transformText)
        transformText.parent = imagePlane
        transformText.location = ((1, 0, 0))
        transformText.scale = .15, .15, 1
        add_constraints(transformText, 'Text')

        #add text next to plane saying which extra was picked
        extraTextName = imagePlane.name+' Extra'
        extraTextData = bpy.data.curves.new(name=extraTextName, type='FONT')
        extraText = bpy.data.objects.new(name=extraTextName, object_data=extraTextData)
        generatorScene.objects.link(extraText)
        extraText.parent = imagePlane
        extraText.location = ((1, -.25, 0))
        extraText.scale = .15, .15, 1
        add_constraints(extraText, 'Text')

    #add text next to plane saying how long the slide is
    lengthTextName = imagePlane.name+' Length'
    lengthTextData = bpy.data.curves.new(name=lengthTextName, type='FONT')
    lengthText = bpy.data.objects.new(name=lengthTextName, object_data=lengthTextData)
    generatorScene.objects.link(lengthText)
    lengthText.parent = imagePlane
    lengthText.location = ((1, .25, 0))
    lengthText.scale = .15, .15, 1
    add_constraints(lengthText, 'Text')

    #create group and add everything to it
    imageGroup = bpy.data.groups.new(imagePlane.name)
    imageGroup.objects.link(imagePlane)
    imageGroup.objects.link(indexText)
    imageGroup.objects.link(lengthText)
    if not video:
        imageGroup.objects.link(targetEmpty)
        imageGroup.objects.link(transformText)
        imageGroup.objects.link(extraText)
        imageGroup.objects.link(viewEmpty)

    #additional settings
    imagePlane.select = True
    bpy.context.scene.objects.active = imagePlane
    imagePlane.slideshow.length = slideshowSlideLength
    imagePlane.slideshow.index = imageNumber + 1
    if not video:
        imagePlane.slideshow.transform = transform['name']
        imagePlane.slideshow.extra = extra
        imagePlane.slideshow.extratexture = extraTexture
        
    return imagePlane


def add_constraints(constraintObject, constraintType):
    #This function handles adding constraints to the various objects of the slideshow generator scene
    bpy.context.scene.objects.active = constraintObject
    rotationConstraint = constraintObject.constraints.new(type='LIMIT_ROTATION')
    locationConstraint = constraintObject.constraints.new(type='LIMIT_LOCATION')
    scaleConstraint = constraintObject.constraints.new(type='LIMIT_SCALE')
    
    #set up rotation constraint
    rotationConstraint.use_limit_x = True
    rotationConstraint.use_limit_y = True
    if (constraintType != 'View'):
        rotationConstraint.use_limit_z = True
    
    #set up local location constraint
    constraintObject.constraints[1].owner_space = 'LOCAL'
    if (constraintType == 'Target' or constraintType == 'View'):
        locationConstraint.min_x = -.888888889
        locationConstraint.max_x = .888888889
        locationConstraint.min_y = -.5
        locationConstraint.max_y = .5
    else:
        locationConstraint.min_x = constraintObject.location[0]
        locationConstraint.max_x = constraintObject.location[0]
        locationConstraint.min_y = constraintObject.location[1]
        locationConstraint.max_y = constraintObject.location[1]
    locationConstraint.use_max_x = True
    locationConstraint.use_min_x = True
    if (constraintType != 'Plane'):
        locationConstraint.use_max_y = True
        locationConstraint.use_min_y = True
    locationConstraint.use_max_z = True
    locationConstraint.use_min_z = True
    
    #set up limit scale constraint
    scaleConstraint.min_x = constraintObject.scale[0]
    scaleConstraint.max_x = constraintObject.scale[0]
    scaleConstraint.min_y = constraintObject.scale[1]
    scaleConstraint.max_y = constraintObject.scale[1]
    scaleConstraint.min_z = constraintObject.scale[2]
    scaleConstraint.max_z = constraintObject.scale[2]
    if (constraintType != 'View'):
        scaleConstraint.use_min_x = True
        scaleConstraint.use_max_x = True
        scaleConstraint.use_min_y = True
        scaleConstraint.use_max_y = True
        scaleConstraint.use_min_z = True
        scaleConstraint.use_max_z = True
    
    #set up text world location limit
    if (constraintType == 'Text'):
        limitLocationConstraint = constraintObject.constraints.new(type='LIMIT_LOCATION')
        limitLocationConstraint.owner_space = 'WORLD'
        limitLocationConstraint.use_max_x = True
        limitLocationConstraint.use_min_x = True
        limitLocationConstraint.min_x = constraintObject.location[0]
        limitLocationConstraint.max_x = constraintObject.location[0]


def create_slideshow_slide(imagePlane, i, generatorScene, slideshowScene, imageSceneStart, images, previousImageClip):
    #This function makes the slideshow image scene and adds it to the sequencer of the slideshowScene scene
    imageSceneName = imagePlane.name

    if not slideshowScene.sequence_editor:
        slideshowScene.sequence_editor_create()

    if not imagePlane.slideshow.videofile:
        print('Generating scene for: '+imageSceneName)
        if (bpy.data.scenes.find(imageSceneName) != -1):
            bpy.data.scenes.remove(bpy.data.scenes[imageSceneName])
        imageScene = create_scene(generatorScene, imageSceneName)
        
        #Set up imageScene render settings
        imageSceneFrames = (imageScene.render.fps * imagePlane.slideshow.length)
        imageScene.frame_end = imageSceneFrames
        
        bpy.context.screen.scene = generatorScene
        bpy.ops.object.select_all(action='DESELECT')
        imagePlane.select = True
        targetEmpty = generatorScene.objects[imagePlane.slideshow.target]
        targetEmpty.select = True
        viewEmpty = generatorScene.objects[imagePlane.slideshow.view]
        viewEmpty.select = True
        bpy.ops.object.make_links_scene(scene=imageScene.name)

        #set scene
        bpy.context.screen.scene = imageScene
        
        #create transforms
        bpy.ops.object.select_all(action='DESELECT')
        imageScene.cursor_location = (0.0, 0.0, 0.0)
        bpy.ops.object.empty_add()
        transformEmpty = bpy.context.scene.objects.active
        transformindex = get_transform(imagePlane.slideshow.transform)
        if (transformindex >= 0):
            transform = transforms[transformindex]
        else:
            transform = transforms[0]
        imageScene.frame_current = 1
        transformEmpty.name = transform['name']
        transformEmpty.animation_data_create()
        transformAction = bpy.data.actions.new(transform['name'])
        transformEmpty.animation_data.action = transformAction
        zoffset = 1.93
        transformEmpty.location = ((0, 0, zoffset))
        if ('zLoc' in transform.keys()):
            locations = len(transform['zLoc'])
            fcurve = transformAction.fcurves.new('location', index=2)
            for index, location in enumerate(transform['zLoc']):
                if (locations == 1):
                    pointx = 1
                else:
                    pointx = ((index * imageSceneFrames)/(locations - 1)) + 1
                pointy = transform['zLoc'][index][0] + zoffset
                pointsize = (transform['zLoc'][index][1]) * (imageSceneFrames / locations / 2)
                point = fcurve.keyframe_points.insert(frame=pointx, value=pointy)
                point.handle_left_type = 'FREE'
                point.handle_right_type = 'FREE'
                point.handle_left = ((pointx - pointsize, pointy))
                point.handle_right = ((pointx + pointsize, pointy))
        if ('zRot' in transform.keys()):
            locations = len(transform['zRot'])
            fcurve = transformAction.fcurves.new('rotation_euler', index=2)
            for index, location in enumerate(transform['zRot']):
                if (locations == 1):
                    pointx = 1
                else:
                    pointx = ((index * imageSceneFrames)/(locations - 1)) + 1
                pointy = 3.14159265358979 * (transform['zRot'][index][0]) / 180
                pointsize = (transform['zRot'][index][1]) * (imageSceneFrames / locations / 2)
                point = fcurve.keyframe_points.insert(frame=pointx, value=pointy)
                point.handle_left_type = 'FREE'
                point.handle_right_type = 'FREE'
                point.handle_left = ((pointx - pointsize, pointy))
                point.handle_right = ((pointx + pointsize, pointy))
        if ('xLoc' in transform.keys()):
            locations = len(transform['xLoc'])
            fcurve = transformAction.fcurves.new('location', index=0)
            for index, location in enumerate(transform['xLoc']):
                if (locations == 1):
                    pointx = 1
                else:
                    pointx = ((index * imageSceneFrames)/(locations - 1)) + 1
                pointy = transform['xLoc'][index][0]
                pointsize = (transform['xLoc'][index][1]) * (imageSceneFrames / locations / 2)
                point = fcurve.keyframe_points.insert(frame=pointx, value=pointy)
                point.handle_left_type = 'FREE'
                point.handle_right_type = 'FREE'
                point.handle_left = ((pointx - pointsize, pointy))
                point.handle_right = ((pointx + pointsize, pointy))
        if ('influence' in transform.keys()):
            constraint = transformEmpty.constraints.new(type='COPY_LOCATION')
            constraint.use_z = False
            constraint.target = targetEmpty
            constraint.influence = 0
            locations = len(transform['influence'])
            fcurve = transformAction.fcurves.new('constraints[0].influence')
            for index, location in enumerate(transform['influence']):
                if (locations == 1):
                    pointx = 1
                else:
                    pointx = ((index * imageSceneFrames)/(locations - 1)) + 1
                pointy = transform['influence'][index][0]
                pointsize = (transform['influence'][index][1]) * (imageSceneFrames / locations / 2)
                point = fcurve.keyframe_points.insert(frame=pointx, value=pointy)
                point.handle_left_type = 'FREE'
                point.handle_right_type = 'FREE'
                point.handle_left = ((pointx - pointsize, pointy))
                point.handle_right = ((pointx + pointsize, pointy))
            
        #set up camera location scale and animation
        bpy.context.scene.cursor_location = (0.0, 0.0, 0.0)
        bpy.ops.object.camera_add()
        camera = bpy.context.scene.objects.active
        camera.parent = transformEmpty
        imageScene.camera = camera
        camera.data.dof_object = imagePlane
        
        #add cameraScale empty that will scale the transform and camera
        bpy.ops.object.empty_add()
        cameraScale = bpy.context.scene.objects.active
        cameraScale.parent = imagePlane
        transformEmpty.parent = cameraScale
        cameraScale.location = viewEmpty.location
        cameraScale.rotation_euler = viewEmpty.rotation_euler
        cameraScaleValue = (viewEmpty.scale[1] * 2)
        cameraScale.scale = ((cameraScaleValue, cameraScaleValue, cameraScaleValue))
        
        #run extras script
        if imagePlane.slideshow.extra != 'None':
            extra = get_extra(imagePlane.slideshow.extra)
            if extra:
                try:
                    folder = os.path.split(extra)[0]
                    file = os.path.splitext(os.path.split(extra)[1])[0]
                    sys.path.insert(0, folder)
                    script = __import__(file)
                    sys.path.remove(folder)
                    image = load_image(bpy.path.abspath(imagePlane.slideshow.extratexture))
                    script.extra(imageScene=imageScene, imagePlane=imagePlane, targetEmpty=targetEmpty, camera=camera, extraAmount=imagePlane.slideshow.extraamount, extraText=imagePlane.slideshow.extratext, extraTexture=image)
                    del script
                except ImportError:
                    pass
        imageScene.update()

        #Add the image scene to the slideshow scene
        clip = slideshowScene.sequence_editor.sequences.new_scene(scene=imageScene, name=imageScene.name, channel=((i % 2) + 1), frame_start=imageSceneStart)

    else:
        print('Importing Clip: '+imageSceneName)
        #Import video clip
        clip = slideshowScene.sequence_editor.sequences.new_movie(filepath=imagePlane.slideshow.videofile, name=imagePlane.name, channel=((i % 2) + 1), frame_start=imageSceneStart)
        
        #Trim video clip
        offset = int(imagePlane.slideshow.videooffset * slideshowScene.render.fps)
        clip.frame_offset_start = offset
        clip.frame_start = imageSceneStart - offset
        imageSceneFrames = int(imagePlane.slideshow.length * slideshowScene.render.fps)
        clip.frame_final_end = clip.frame_final_start + imageSceneFrames
        
        #Import audio portion of video clip
        if imagePlane.slideshow.videoaudio:
            audioclip = slideshowScene.sequence_editor.sequences.new_sound(filepath=imagePlane.slideshow.videofile, name=imagePlane.name, channel=((i % 2) + 1) + 3, frame_start=imageSceneStart)
            if audioclip.frame_duration == 0:
                slideshowScene.sequence_editor.sequences.remove(audioclip)
                print('No Audio Found For This Clip')

            else:
                #Trim audio clip
                audioclip.frame_offset_start = offset
                audioclip.frame_start = imageSceneStart - offset
                audioclip.frame_final_end = audioclip.frame_final_start + imageSceneFrames
            
                bpy.context.screen.scene = slideshowScene

                #Add fadein/out to audio clip
                slideshowScene.frame_current = imageSceneStart
                audioclip.volume = 0
                audioclip.keyframe_insert(data_path='volume')
                slideshowScene.frame_current = imageSceneStart + slideshowScene.slideshowCrossfadeLength
                audioclip.volume = 1
                audioclip.keyframe_insert(data_path='volume')
                slideshowScene.frame_current = audioclip.frame_final_start + imageSceneFrames - slideshowScene.slideshowCrossfadeLength
                audioclip.keyframe_insert(data_path='volume')
                slideshowScene.frame_current = audioclip.frame_final_start + imageSceneFrames
                audioclip.volume = 0
                audioclip.keyframe_insert(data_path='volume')
        
    bpy.context.screen.scene = slideshowScene

    #Add fade in
    if i == 0:
        clip.blend_type = 'REPLACE'
        slideshowScene.frame_current = 1
        clip.blend_alpha = 0
        clip.keyframe_insert(data_path='blend_alpha')
        slideshowScene.frame_current = generatorScene.slideshowCrossfadeLength
        clip.blend_alpha = 1
        clip.keyframe_insert(data_path='blend_alpha')
        
    #Add crossfades
    if generatorScene.slideshowCrossfadeLength > 0:
        if (previousImageClip):
            firstSequence = previousImageClip
            secondSequence = clip
            crossfade = slideshowScene.sequence_editor.sequences.new_effect(name=previousImageClip.name+'to'+clip.name, channel=3, frame_start=secondSequence.frame_final_start, frame_end=firstSequence.frame_final_end, type='CROSS', seq1=firstSequence, seq2=secondSequence)
            crossfade.frame_still_end = firstSequence.frame_final_end       #apparently needed since the frame_end doesnt get set in the previous function...
    
    #Add fade out
    if (i == (len(images) - 1)):
        clip.blend_type = 'REPLACE'
        slideshowScene.frame_current = imageSceneStart + imageSceneFrames
        clip.blend_alpha = 0
        clip.keyframe_insert(data_path='blend_alpha')
        slideshowScene.frame_current = imageSceneStart + imageSceneFrames - generatorScene.slideshowCrossfadeLength
        clip.blend_alpha = 1
        clip.keyframe_insert(data_path='blend_alpha')
        slideshowScene.frame_current = 1
    
    return clip


def extras_path():
    #This function will return the path to the Extras files
    for path in bpy.utils.script_paths():
        if os.path.exists(path+"\\addons\\slideshow\\Extras"):
            return path+"\\addons\\slideshow\\Extras\\*"
        if os.path.exists(path+"\\addons\\Extras"):
            return path+"\\addons\\Extras\\*"
    return os.path.split(bpy.data.filepath)[0]+"\\Extras\\*"


def list_extras():
    #This function will return a list of Extras found in the extras path location
    extras = []
    extraspath = extras_path()
    extrafiles = glob.glob(extraspath)
    for file in extrafiles:
        if (os.path.splitext(file)[1] == '.py'):
            extras.append(os.path.splitext(os.path.split(file)[1])[0])
    return extras


def get_extra(filename):
    #This function will return the full path to an extra from the filename
    extra = None
    extraspath = extras_path()
    extrafiles = glob.glob(extraspath)
    for file in extrafiles:
        if (os.path.splitext(os.path.split(file)[1])[0] == filename):
            extra = file
    return extra


def get_transform(name):
    #This function will return a transform from the name
    for index, transform in enumerate(transforms):
        if (transform['name'] == name):
            return index
    return -1


def update_slide_length(self, context):
    #Callback for SlideshowImage.length to update the length display text
    currentScene = bpy.data.scenes['Slideshow Generator']
    lengthText = currentScene.objects[self.name+" Length"]
    if self.videofile:
        maxseconds = self.videomaxlength / currentScene.render.fps
        maxallowed = round(maxseconds-round(self.videooffset,2), 2)
        if round(self.length,2) > maxallowed:
            self.length = maxallowed
    lengthText.data.body = "Length: "+str(round(self.length, 2))+" Seconds"


def update_offset(self, context):
    #Callback for SlideshowImage.videooffset to update the image texture's offset
    currentScene = bpy.data.scenes['Slideshow Generator']
    imagePlane = currentScene.objects[self.name]
    texture = imagePlane.material_slots[0].material.texture_slots[0].texture
    maxlength = round(self.videomaxlength / currentScene.render.fps, 2) - 1
    if round(self.videooffset,2) > maxlength:
        self.videooffset = maxlength
        offset = int(self.length * currentScene.render.fps)
    else:
        offset = int(self.videooffset * currentScene.render.fps)
    texture.image_user.frame_offset = offset
    update_slide_length(self, context)


def update_extra(self, context):
    #Callback for SlideshowImage.extra to update the extra display text
    try:
        currentScene = bpy.data.scenes['Slideshow Generator']
        extraText = currentScene.objects[self.name+" Extra"]
        extraText.data.body = "Extra: "+self.extra
    except:
        pass


def update_transform(self, context):
    #Callback for SlideshowImage.transform to update the transform display text
    try:
        currentScene = bpy.data.scenes['Slideshow Generator']
        transformText = currentScene.objects[self.name+" Transform"]
        transformText.data.body = "Transform: "+self.transform
    except:
        pass


def update_index(self, context):
    #Callback for SlideshowImage.index to update the index display text, and the position of the image plane
    currentScene = bpy.data.scenes['Slideshow Generator']
    imagePlane = currentScene.objects[self.name]
    position = -self.index
    imagePlane.location = ((0.0, position, 0.0))
    indexText = currentScene.objects[self.name+" Index"]
    indexText.data.body = str(self.index + 1)


def list_slides(scene):
    #This function returns an unordered list of the slideshow images in the generator scene
    slides = []
    for object in scene.objects:
        if (object.slideshow.name == object.name):
            slides.append(object)
    return slides


def slideshow_length():
    slides = list_slides(bpy.context.scene)
    length = 0.0
    crossfadeLength = (bpy.context.scene.slideshowCrossfadeLength / bpy.context.scene.render.fps)
    for index, slide in enumerate(slides):
        length = length + slide.slideshow.length
        if index > 0:
            length = length - crossfadeLength
    return length


def update_order(mode='none'):
    #This function is called by the modal autoupdate and by various operators.  It arranges and positions the slides in the way they need to be.
    #   Setting 'mode' to 'none' will place the slides in the exact positions they should be, and update their index values
    #   Setting 'mode' to 'random' will rearrange the slides in a random order
    #   Setting 'mode' to 'alphabetical' will arrange the slides in alphabetical order
    #   Setting 'mode' to 'reverse' will arrange the slides in reverse alphabetical order
    
    currentScene = bpy.context.scene
    slides = list_slides(currentScene)
    
    if mode == 'none':
        #Sort slides by their current location only
        changed = False
        oldorder = []
        for slide in slides:
            loc = -slide.location[1]
            oldorder.append([loc, slide])
        neworder = sorted(oldorder, key=lambda x: x[0])
        for i, slide in enumerate(neworder):
            if slide[0] != i:
                slide[1].slideshow.index = i
                changed = True
        if changed:
            currentScene.update()
    else:
        #Resort the slides in a specific way
        
        #Solt the slides by index to start with
        slides.sort(key=lambda x: x.slideshow.index)
        
        #Make a list of only the slides that are not locked
        unfrozen_indices, unfrozen_subset = zip(*[(i, e) for i, e in enumerate(slides) if not e.slideshow.lockposition])
        unfrozen_indices = list(unfrozen_indices)
        unfrozen_subset = list(unfrozen_subset)

        if mode == 'random':
            #Shuffle the index values of the unlocked slides
            random.shuffle(unfrozen_indices)
        elif mode == 'alphabetical':
            unfrozen_subset.sort(key=lambda x: x.slideshow.name)
        elif mode == 'reverse':
            unfrozen_subset.sort(key=lambda x: x.slideshow.name)
            unfrozen_subset.reverse()

        #Recombine the sorted indexes and slides
        for i, e in zip(unfrozen_indices, unfrozen_subset):
            slides[i] = e

        #Reenter the slide indexes
        for i, slide in enumerate(slides):
            slide.slideshow.index = i


class SlideshowExtraTexturePreset(bpy.types.PropertyGroup):
    #A property group for a texture preset for the extra scenes
    name = bpy.props.StringProperty(
        name="Texture Name",
        default="None")
    path = bpy.props.StringProperty(
        name="Texture Path",
        default="None")


class SlideshowImage(bpy.types.PropertyGroup):
    #A property group that contains the information needed for a slideshow image.  This is appended to a slide image plane object.
    name = bpy.props.StringProperty(
        name="Image Name",
        default="None")
    transform = bpy.props.StringProperty(
        name="Transform Type Name",
        default="None",
        update=update_transform)
    length = bpy.props.FloatProperty(
        name="Slide Length (Seconds)", 
        default=4, 
        min=1,
        update=update_slide_length,
        description="Slide Length In Seconds")
    videooffset = bpy.props.FloatProperty(
        name="Video Offset (Seconds)",
        default=0,
        min=0,
        description="Video Offset In Seconds",
        update=update_offset)
    videoaudio = bpy.props.BoolProperty(
        name="Enable Audio",
        default=True,
        description="Import Audio Track When Importing Video")
    videomaxlength = bpy.props.IntProperty(
        name="Video Maximum Length",
        default=0)
    videofile = bpy.props.StringProperty(
        name="Video Filename",
        default="")
    extra = bpy.props.StringProperty(
        name="Extra Type", 
        default="None",
        update=update_extra)
    imageplane = bpy.props.StringProperty(
        name="Image Plane Name", 
        default="None")
    target = bpy.props.StringProperty(
        name="Target Empty Name", 
        default="None")
    view = bpy.props.StringProperty(
        name="View Window Empty Name", 
        default="None")
    extraamount = bpy.props.FloatProperty(
        name="Amount For The Extra Scene",
        default=0.5,
        max=1,
        min=0,
        description="Determines the power of the effect for the extra scene - 0.5 is average")
    extratext = bpy.props.StringProperty(
        name="Text For The Extra Scene",
        default="None",
        description="Used for extra scenes with a text object")
    extratexture = bpy.props.StringProperty(
        name="Texture For The Extra Scene",
        default="None",
        description="Used for extra scenes with a video background or overlay")
    index = bpy.props.IntProperty(
        name="Image Index",
        default=0,
        update=update_index)
    lockposition = bpy.props.BoolProperty(
        name="Lock Position",
        default=False)
    locktransform = bpy.props.BoolProperty(
        name="Lock Transform",
        default=False)
    lockextra = bpy.props.BoolProperty(
        name="Lock Extra",
        default=False)
    locklength = bpy.props.BoolProperty(
        name="Lock Length",
        default=False)


class SlideshowVSEPanel(bpy.types.Panel):
    #This is the panel that is visible in the VSE of the 'Slideshow' scene.
    bl_label = "Slideshow Generator"
    bl_space_type = 'SEQUENCE_EDITOR'
    bl_region_type = 'UI'
    @classmethod
    def poll(self, context):
        #Check if in slideshow scene
        if context.scene.name == 'Slideshow':
            try:
                if len(context.scene.sequence_editor.sequences_all) > 0:
                    return True
                else:
                    return False
            except:
                return False
        else:
            return False
    
    def draw(self, context):
        layout = self.layout
        layout.operator('slideshow.preview_mode', text="Apply 50% Render Size").mode = 'halfsize'
        layout.operator('slideshow.preview_mode', text="Apply 100% Render Size").mode = 'fullsize'
        layout.operator('slideshow.preview_mode', text="Disable Antialiasing").mode = 'aliasing'
        layout.operator('slideshow.preview_mode', text="Enable Antialiasing").mode = 'antialiasing'
        layout.operator('slideshow.gotogenerator', text="Return To The Generator Scene")


class SlideshowGotoGenerator(bpy.types.Operator):
    #This operator will attempt to go back to the slideshow scene and default screen mode so the slides can be edited and re-generated
    bl_idname = 'slideshow.gotogenerator'
    bl_label = "Return To The Generator Scene"
    
    def execute(self, context):
        if "Slideshow Generator" in bpy.data.scenes:
            generatorScene = bpy.data.scenes['Slideshow Generator']
            bpy.context.screen.scene = generatorScene
            try:
                bpy.context.window.screen = bpy.data.screens['Default']
                bpy.data.screens['Default'].scene = generatorScene
            except KeyError:
                pass
        return{'FINISHED'}


class SlideshowPreviewMode(bpy.types.Operator):
    #This operator will enable or disable 'preview' mode on all scene strips in the sequencer.
    #'mode' can be set to:
    #   halfsize - Set the rendering to 50% size
    #   fullsize - Set the rendering to 100% size
    #   aliasing - Disable antialiasing
    #   antialiasing - Enable antialiasing
    bl_idname = 'slideshow.preview_mode'
    bl_label = 'Enable or Disable Preview On All Scenes'
    
    mode = bpy.props.StringProperty()
    
    def execute(self, context):
        sequences = context.scene.sequence_editor.sequences_all
        for sequence in sequences:
            if sequence.type == 'SCENE':
                scene = sequence.scene
                if self.mode == 'halfsize':
                    scene.render.resolution_percentage = 50
                elif self.mode == 'fullsize':
                    scene.render.resolution_percentage = 100
                elif self.mode == 'aliasing':
                    scene.render.use_antialiasing = False
                elif self.mode == 'antialiasing':
                    scene.render.use_antialiasing = True
        return{'FINISHED'}


class SlideshowPanel(bpy.types.Panel):
    #This is the main configuration panel for the slideshow generator and generator creator
    bl_label = "Slideshow Generator"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "scene"
    
    def draw(self, context):
        layout = self.layout
        currentScene = bpy.context.scene
        if currentScene.name == 'Slideshow Generator':
            #Panel is in slideshow generator mode
            slides = list_slides(currentScene)
            row = layout.row()
            if len(slides):
                layout.operator_context = 'INVOKE_SCREEN' #needed to solve issues with bpy.context.scene not updating
                
                row.operator('slideshow.create')
                
                row = layout.row()
                #Calculate and display slideshow length
                slideshowSeconds = slideshow_length()
                if slideshowSeconds > 60:
                    slideshowMinutes, slideshowExtraSeconds = divmod(slideshowSeconds, 60)
                    slideshowLength = str(int(slideshowMinutes))+" Minutes, "+str(int(slideshowExtraSeconds))+" Seconds"
                else:
                    slideshowLength = str(round(slideshowSeconds, 2)) + " Seconds"
                row.label(str(len(list_slides(currentScene))) + " Slides, Slideshow Length: "+slideshowLength)
            else:
                row.label("No Slides Found")

            row = layout.row()
            row.prop(context.scene, "slideshowCrossfadeLength")
            
            row = layout.row()
            box = row.box()
            
            row = box.row()
            row.prop(context.scene, "slideshowAudioEnabled", text="Enable Audio Track")
            
            row = box.row()
            split = row.split(percentage=.9, align=True)
            split.prop(context.scene, "slideshowAudioTrack", text="Audio Track")
            split.operator('slideshow.open_audio', text="", icon='FILESEL')
            if not context.scene.slideshowAudioEnabled:
                row.enabled = False
            
            row = box.row()
            row.prop(context.scene, "slideshowAudioLoopFade")
            row.prop(context.scene, "slideshowAudioFadeLength")
            if not context.scene.slideshowAudioEnabled:
                row.enabled = False
            
            row = layout.row()
            row.separator()
            
            if len(slides):
                #Sorting options
                row = layout.row(align=True)
                row.label('Sort:')
                row.operator('slideshow.update_order', text='Randomize').mode = 'random'
                row.operator('slideshow.update_order', text='Alphabetical').mode = 'alphabetical'
                row.operator('slideshow.update_order', text='Reverse Alpha').mode = 'reverse'
                
                row = layout.row(align=True)
                row.label('Randomize:')
                row.operator('slideshow.apply_transform', text='Transforms').transform = 'Random'
                row.operator('slideshow.apply_extra', text='Extras').extra = 'Random'

            row = layout.row(align=True)
            row.operator('slideshow.add_slide')

            if bpy.context.selected_objects:
                row.operator('slideshow.delete_slide')
                
                row = layout.row()
                row.separator()
                
                selected = bpy.context.scene.objects.active
                if selected.slideshow.name != "None":
                    #If a slide image is selected, display configuration options
                    currentSlide = selected.slideshow
                    
                    box = layout.box()
                    
                    row = box.row()
                    row.label("Image: "+currentSlide.name)
                    
                    row = box.row()
                    split = row.split(percentage=.5)
                    subsplit = split.split(align=True)
                    subsplit.label(""+str(currentSlide.index + 1))
                    subsplit.operator("slideshow.move_slide", text="", icon="REW").move = "beginning"
                    subsplit.operator("slideshow.move_slide", text="", icon="PLAY_REVERSE").move = "backward"
                    subsplit.operator("slideshow.move_slide", text="", icon="PLAY").move = "forward"
                    subsplit.operator("slideshow.move_slide", text="", icon="FF").move = "end"
                    split = split.split()
                    split.prop(currentSlide, "lockposition")
                    
                    innerbox = box.box()
                    
                    row = innerbox.row()
                    row.prop(currentSlide, "length")
                    
                    row = innerbox.row()
                    row.operator('slideshow.apply_slide_length', text="Apply To Selected").mode = 'Selected'
                    row.operator('slideshow.apply_slide_length').mode = 'None'
                    row.prop(currentSlide, "locklength")
                    
                    if not currentSlide.videofile:
                        innerbox = box.box()
                        row = innerbox.row()
                        row.label("Transform: "+currentSlide.transform)
                        row = innerbox.row()
                        row.menu('slideshow.transforms_menu', text="Change Transform")
                        row = innerbox.row()
                        row.operator('slideshow.apply_transform', text="Apply To Selected").transform = 'Selected'
                        row.operator('slideshow.apply_transform').transform = 'None'
                        row.prop(currentSlide, "locktransform")
                        
                        innerbox = box.box()
                        row = innerbox.row()
                        row.label("Extra: "+currentSlide.extra)
                        row = innerbox.row()
                        row.menu('slideshow.extra_menu', text="Change Extra")
                        row = innerbox.row()
                        row.operator('slideshow.apply_extra', text="Apply To Selected").extra = 'Selected'
                        row.operator('slideshow.apply_extra').extra = 'None'
                        row.prop(currentSlide, "lockextra")
                        row = innerbox.row()
                        row.separator()
                        row = innerbox.row()
                        row.prop(currentSlide, "extratext", text='Extra Text')
                        row = innerbox.split(percentage=0.8)
                        split = row.split(percentage=.9, align=True)
                        split.prop(currentSlide, "extratexture", text='Extra Texture')
                        split.operator('slideshow.open_extra_texture', text="", icon='FILESEL').target = 'slide'
                        split = row.split(percentage=.5, align=True)
                        split.operator('slideshow.add_extra_texture').texture = currentSlide.extratexture
                        split.operator('slideshow.remove_extra_texture').texture = currentSlide.extratexture
                        row = innerbox.row()
                        row.menu('slideshow.extra_texture_menu', text="Apply Extra Texture Preset")
                        row = innerbox.row()
                        row.prop(currentSlide, "extraamount", text='Extra Amount')
                    
                    else:
                        innerbox = box.box()
                        row = innerbox.row()
                        row.prop(currentSlide, "videooffset", text='Video Offset')
                        row = innerbox.row()
                        row.prop(currentSlide, "videoaudio", text='Import Audio From Video File')
                        
                    row = layout.row()
                    row.separator()
                                        
                    row = layout.box()
                    row.label("Drag an image up or down to rearrange it in the timeline.")
                
                elif 'View' in selected.name:
                    #The view area rectangle is probably selected
                    row = layout.box()
                    row.label("The rectangle is the area the camera will see.")
                    row.label("It can be scaled down to crop the image,")
                    row.label("or rotated to rotate the camera,")
                    row.label("or moved to focus on a specific area.")
                
                elif 'Target' in selected.name:
                    #The zoom to target is probably selected
                    row = layout.box()
                    row.label("The cross is the target for transforms like zoom in.")
                    row.label("It can be moved around the image.")
                
                else:
                    #Something else is selected
                    row = layout.box()
                    row.label("Select An Image To Customize It")
            
            else:
                #Nothing is selected
                row = layout.row()
                row.separator()
                row = layout.box()
                row.label("Select An Image To Customize It")

        else:
            #Panel is in create generator mode
            row = layout.row()
            row.prop(context.scene, "slideshowImageDirectory")
            
            row = layout.row()
            row.prop(context.scene, "slideshowSlideLength")
            
            row = layout.row()
            row.operator('slideshow.generator')
            #get list of images in slideshowImageDirectory, gray out generator button if none found
            imageTypes = list(bpy.path.extensions_image)
            imageList = []
            for type in imageTypes:
                imageList.extend(glob.glob(bpy.path.abspath(bpy.context.scene.slideshowImageDirectory)+'*'+type))
            videoTypes = list(bpy.path.extensions_movie)
            for type in videoTypes:
                imageList.extend(glob.glob(bpy.path.abspath(bpy.context.scene.slideshowImageDirectory)+'*'+type))
            if not imageList:
                row.enabled = False
                
                row = layout.row()
                row.label("Image Directory Invalid Or Empty")
            else:
                row.enabled = True
                
                row = layout.row()
                row.label(str(len(imageList))+" Images In Directory")
            
            row = layout.row()
            row.separator()
            
            box = layout.box()
            
            row = box.row()
            row.menu('slideshow.transforms_menu', text="Toggle Transforms")
            
            row = box.row()
            row.menu('slideshow.extra_menu', text="Toggle Extras")
            
            row = layout.row()
            row.separator()
            
            box = layout.box()
            
            row = box.split(percentage=0.8)
            split = row.split(percentage=.9, align=True)
            split.prop(context.scene, "slideshowExtraTexture", text='Extra Texture')
            split.operator('slideshow.open_extra_texture', text="", icon='FILESEL').target = 'scene'
            split = row.split(percentage=.5, align=True)
            split.operator('slideshow.add_extra_texture').texture = context.scene.slideshowExtraTexture
            split.operator('slideshow.remove_extra_texture').texture = context.scene.slideshowExtraTexture

            row = box.row()
            row.menu('slideshow.extra_texture_menu', text="Extra Texture Presets")


class SlideshowMoveSlide(bpy.types.Operator):
    #Operator for moving the current active slideshow slide up or down in the generator scene.
    #'move' can be:
    #   'forward' for moving it one space forward
    #   'backward' for moving it one space back
    #   'beginning' for moving it to the start of the slideshow
    #   'end' for moving it to the end of the slideshow
    bl_idname = 'slideshow.move_slide'
    bl_label = "Move The Active Slide"
    
    move = bpy.props.StringProperty()
    
    def execute(self, context):
        selected = bpy.context.scene.objects.active
        slide = selected.slideshow
        slides = list_slides(context.scene)
        if self.move == "forward":
            selected.location[1] = selected.location[1] - 1.1
            update_order()
        elif self.move == "backward":
            selected.location[1] = selected.location[1] + 1.1
            update_order()
        elif self.move == "beginning":
            slide.index = -1
        elif self.move == "end":
            slide.index = len(slides) + 1
        return{'FINISHED'}        


class SlideshowOpenExtraTexture(bpy.types.Operator):
    #This operator will open a filebrowser to select the scene.slideshowExtraTexture variable
    bl_idname = 'slideshow.open_extra_texture'
    bl_label = 'Browse For An Image Or Video File'
            
    #The browsed to file is stored in this variable
    filepath = bpy.props.StringProperty()
    
    target = bpy.props.StringProperty()

    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

    def draw(self, context):
        context.space_data.params.use_filter = True
        context.space_data.params.use_filter_image = True
        context.space_data.params.use_filter_movie = True
        context.space_data.params.use_filter_folder = True
        
    def execute(self, context):
        #This function is called when the file browser dialog is closed with an ok
        if self.target == 'slide':
            currentScene = context.scene
            currentSlide = currentScene.objects.active
            currentSlide.slideshow.extratexture = self.filepath
        else:
            currentScene = context.scene
            currentScene.slideshowExtraTexture = self.filepath
        
        return{'FINISHED'}


class SlideshowOpenAudio(bpy.types.Operator):
    #This operator will open a filebrowser to select the scene.slideshowAudioTrack variable
    bl_idname = 'slideshow.open_audio'
    bl_label = 'Browse For An Audio File'
            
    #The browsed to file is stored in this variable
    filepath = bpy.props.StringProperty()

    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

    def draw(self, context):
        context.space_data.params.use_filter = True
        context.space_data.params.use_filter_sound = True
        context.space_data.params.use_filter_folder = True
        
    def execute(self, context):
        #This function is called when the file browser dialog is closed with an ok
        generatorScene = bpy.data.scenes['Slideshow Generator']
        generatorScene.slideshowAudioTrack = self.filepath
        
        return{'FINISHED'}


class SlideshowAddSlide(bpy.types.Operator):
    #This operator will add a new slide to the slideshow generator scene.
    bl_idname = 'slideshow.add_slide'
    bl_label = 'Add New Slide(s)'
    
    #The files selected by the browser are stored in this
    files = bpy.props.CollectionProperty(
        name="File Path",
        type=bpy.types.OperatorFileListElement)
        
    #The browsed to directory is stored in this variable
    directory = bpy.props.StringProperty(
        subtype="DIR_PATH")

    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

    def draw(self, context):
        context.space_data.params.use_filter = True
        context.space_data.params.use_filter_image = True
        context.space_data.params.use_filter_movie = True
        context.space_data.params.use_filter_folder = True
        
    def execute(self, context):
        #This function is called when the file browser dialog is closed with an ok
        generatorScene = bpy.data.scenes['Slideshow Generator']
        import os
        
        #iterate through the selected files and attempt to load them into the generator scene
        lastImage = None
        for fileelement in self.files:
            filename = os.path.join(self.directory, fileelement.name)
            if os.path.isfile(filename):
                #The file is a real file
                extension = os.path.splitext(filename)[1].lower()
                if extension in bpy.path.extensions_image:
                    #The file is a known image file type, load it
                    image = load_image(filename)
                    imageNumber = len(list_slides(generatorScene))
                    lastImage = import_slideshow_image(image, imageNumber, generatorScene.slideshowSlideLength, generatorScene, video=False, lastImage=lastImage)
                elif extension in bpy.path.extensions_movie:
                    #The file is a known video file type, load it
                    image = load_image(filename)
                    imageNumber = len(list_slides(generatorScene))
                    lastImage = import_slideshow_image(image, imageNumber, generatorScene.slideshowSlideLength, generatorScene, video=True, lastImage=lastImage)
                else:
                    #oops, this file is not an image or video
                    self.report({'WARNING'}, os.path.split(filename)[1]+' Is Not An Image')
        return{'FINISHED'}


class SlideshowExtraMenu(bpy.types.Menu):
    #This is the popup menu for selecting or toggling extras
    bl_idname = 'slideshow.extra_menu'
    bl_label = 'List of available extras'

    def draw(self, context):
        scene = context.scene
        layout = self.layout
        split = layout.split()
        column = split.column()
        
        #Display the 'None' extra
        if scene.name == "Slideshow Generator":
            #This is the slideshow generator scene, the extra can be selected
            column.operator("slideshow.change_extra", text="None").extra = "None"
        else:
            #This is not the slideshow generator scene, the extra can only be toggled
            column.operator("slideshow.hide_extra", text="None").extra = "None"
        extras = list_extras()
        
        #Iterate through the extras to display the extra name column
        for extra in extras:
            if extra != 'None':
                if scene.name == "Slideshow Generator":
                    #This is the slideshow generator scene, the extra can be selected
                    column.operator("slideshow.change_extra", text=extra).extra = extra
                else:
                    #This is not the slideshow generator scene, the extra can only be toggled
                    column.operator("slideshow.hide_extra", text=extra).extra = extra
        
        column = split.column()
        
        #Display the toggled extra checkboxes
        hidden = scene.slideshowExtrasHidden.split(";")
        
        #Display the 'None' extra checkbox
        if "None" in hidden:
            column.operator("slideshow.hide_extra", text="", icon="CHECKBOX_DEHLT").extra = "None"
        else:
            column.operator("slideshow.hide_extra", text="", icon="CHECKBOX_HLT").extra = "None"
        
        #Iterate through the extras and display the toggle extra checkboxes
        for extra in extras:
            if (extra != 'None'):
                if extra in hidden:
                    #This extra is currently disabled
                    column.operator("slideshow.hide_extra", text="", icon="CHECKBOX_DEHLT").extra = extra
                else:
                    #This extra is currently enabled
                    column.operator("slideshow.hide_extra", text="", icon="CHECKBOX_HLT").extra = extra
        
        #This will toggle all extras on and off
        column.operator("slideshow.hide_all_extras", text="Toggle All")


class SlideshowHideAllExtras(bpy.types.Operator):
    #This operator will enable all extras if none are enabled, otherwise it will disable all
    bl_idname = 'slideshow.hide_all_extras'
    bl_label = 'Toggle Hide All Extras From Randomize Operations'
    bl_description = 'Toggle Hide All Extras From Randomize Operations'
    
    def execute(self, context):
        scene = context.scene
        if len(scene.slideshowExtrasHidden) == 0:
            hidden = list_extras()
            scene.slideshowExtrasHidden = ';'.join(hidden)    
        else:
            scene.slideshowExtrasHidden = ""
        return{'FINISHED'}


class SlideshowHideExtra(bpy.types.Operator):
    #This operator will toggle an extra in the hidden list
    bl_idname = 'slideshow.hide_extra'
    bl_label = 'Hide Extra From Randomize Operations'
    bl_description = 'Hide Extra From Randomize Operations'
    
    extra = bpy.props.StringProperty()
    
    def execute(self, context):
        scene = context.scene
        hidden = scene.slideshowExtrasHidden.split(';')
        if self.extra in hidden:
            hidden.remove(self.extra)
        else:
            hidden.append(self.extra)
        scene.slideshowExtrasHidden = ';'.join(hidden)    
        return{'FINISHED'}


class SlideshowChangeExtra(bpy.types.Operator):
    #This operator will change the listed extra in the currently selected object
    bl_idname = 'slideshow.change_extra'
    bl_label = 'Change Extra'
    
    extra = bpy.props.StringProperty()
    
    def execute(self, context):
        currentScene = bpy.context.scene
        currentImage = currentScene.objects.active
        currentImage.slideshow.extra = self.extra
        return{'FINISHED'}


class SlideshowTransformsMenu(bpy.types.Menu):
    #This is a menu of the transforms and allows for them to be disabled
    bl_idname = 'slideshow.transforms_menu'
    bl_label = 'List of available transforms'
    
    def draw(self, context):
        scene = context.scene
        layout = self.layout
        split = layout.split()
        column = split.column()
        
        #Iterate through transforms and display the transforms name column
        for index, transform in enumerate(transforms):
            if scene.name == "Slideshow Generator":
                #This is the slideshow generator scene, the transforms can be selected
                column.operator("slideshow.change_transform", text=transform['name']).transform = transform['name']
            else:
                #This is not the slideshow generator scene, the transforms can only be toggled
                column.operator("slideshow.hide_transform", text=transform['name']).transform = transform['name']
        
        column = split.column()
        hidden = scene.slideshowTransformsHidden.split(';')
        
        #iterate through transforms and display enable/disable column
        for index, transform in enumerate(transforms):
            if transform['name'] in hidden:
                #This transform is not enabled
                column.operator("slideshow.hide_transform", text="", icon="CHECKBOX_DEHLT").transform = transform['name']
            else:
                #This transform is enabled
                column.operator("slideshow.hide_transform", text="", icon="CHECKBOX_HLT").transform = transform['name']
        column.operator("slideshow.hide_all_transforms", text="Toggle All")


class SlideshowHideAllTransforms(bpy.types.Operator):
    #This operator will enable all transforms if none are enabled, otherwise it will disable all
    bl_idname = 'slideshow.hide_all_transforms'
    bl_label = 'Toggle Hide All Transforms From Randomize Operations'
    bl_description = 'Toggle Hide All Transforms From Randomize Operations'
    
    def execute(self, context):
        scene = context.scene
        if len(scene.slideshowTransformsHidden) == 0:
            hidden = []
            for transform in transforms:
                hidden.append(transform['name'])
            scene.slideshowTransformsHidden = ';'.join(hidden)    
        else:
            scene.slideshowTransformsHidden = ""
        return{'FINISHED'}


class SlideshowHideTransform(bpy.types.Operator):
    #This operator will toggle a transform in the hidden list
    bl_idname = 'slideshow.hide_transform'
    bl_label = 'Hide Transform From Randomize Operations'
    bl_description = 'Hide Transform From Randomize Operations'
    
    transform = bpy.props.StringProperty()
    
    def execute(self, context):
        scene = context.scene
        hidden = scene.slideshowTransformsHidden.split(';')
        if self.transform in hidden:
            hidden.remove(self.transform)
        else:
            hidden.append(self.transform)
        scene.slideshowTransformsHidden = ';'.join(hidden)    
        return{'FINISHED'}


class SlideshowChangeTransform(bpy.types.Operator):
    #This operator will change the listed transform in the currently selected object
    bl_idname = 'slideshow.change_transform'
    bl_label = 'Change Transform'
    
    transform = bpy.props.StringProperty()
    
    def execute(self, context):
        currentScene = bpy.context.scene
        currentImage = currentScene.objects.active
        currentImage.slideshow.transform = self.transform
        return{'FINISHED'}


class SlideshowApplyExtra(bpy.types.Operator):
    #This operator will apply an extra, or randomize extras on all slides that do not have locked extras
    bl_idname = 'slideshow.apply_extra'
    bl_label = 'Apply To All'
    bl_description = 'Applies an extra to all slides'

    extra = bpy.props.StringProperty()

    def execute(self, context):
        update_order()
        currentScene = bpy.context.scene
        slides = list_slides(currentScene)
        slides.sort(key=lambda x: x.slideshow.index)
        extras = list_extras()
        hidden = context.scene.slideshowExtrasHidden.split(";")
        randomized = []
        currentSlide = bpy.context.scene.objects.active
        
        if self.extra == 'Selected':
            objects = currentScene.objects
            for object in objects:
                if object.select:
                    if object.slideshow.name != "None":
                        if not object.slideshow.lockextra:
                            object.slideshow.extra = currentSlide.slideshow.extra
                            object.slideshow.extraamount = currentSlide.slideshow.extraamount
                            object.slideshow.extratexture = currentSlide.slideshow.extratexture
                            object.slideshow.extratext = currentSlide.slideshow.extratext
            
        else:
            #Get list of extras that are not disabled for randomization purposes
            for extra in extras:
                if extra not in hidden and extra != "None":
                    randomized.append(extra)
            if "None" not in hidden:
                randomized.append("None")
            lastextra = ""
            lastextratexture = ""
            
            extratextures = []
            for extraTexturePreset in currentScene.slideshowExtraTexturePresets:
                extratextures.append(extraTexturePreset.path)

            #Iterate through all slides
            for slide in slides:
                if not slide.slideshow.lockextra:
                    #Only check slides that are not locked
                    if self.extra == 'Random':
                        #Apply a random extra
                        if len(randomized) > 0:
                            #Make sure at least one extra is enabled
                            
                            #Randomize extra texture
                            lessrandomized = [x for x in extratextures if x != lastextratexture]
                            if len(lessrandomized) > 0:
                                newextratexture = lessrandomized[random.randint(0, (len(lessrandomized)-1))]
                                slide.slideshow.extratexture = newextratexture
                                
                            #Randomize extra
                            if len(randomized) > 1:
                                #More than one extra is enabled, so dont use the same extra twice in a row
                                lessrandomized = [x for x in randomized if x != lastextra]
                                newextra = lessrandomized[random.randint(0, (len(lessrandomized)-1))]
                                slide.slideshow.extra = newextra
                                slide.slideshow.extraamount = 0.5
                                lastextra = newextra

                            else:
                                #Only one extra is enabled, use it
                                slide.slideshow.extra = randomized[0]
                                slide.slideshow.extraamount = 0.5

                    else:
                        #Apply the extra from currentSlide to the slide
                        slide.slideshow.extra = currentSlide.slideshow.extra
                        slide.slideshow.extraamount = currentSlide.slideshow.extraamount
                        slide.slideshow.extratexture = currentSlide.slideshow.extratexture
                        slide.slideshow.extratext = currentSlide.slideshow.extratext

        return{'FINISHED'}


class SlideshowApplyTransform(bpy.types.Operator):
    #This operator will apply a transform, or randomize transforms on all slides that do not have locked transforms
    bl_idname = 'slideshow.apply_transform'
    bl_label = 'Apply To All'
    bl_description = 'Applies a transform to all slides'
    
    transform = bpy.props.StringProperty()
    
    def execute(self, context):
        update_order()
        currentScene = bpy.context.scene
        currentSlide = currentScene.objects.active
        slides = list_slides(currentScene)
        slides.sort(key=lambda x: x.slideshow.index)
        hidden = context.scene.slideshowTransformsHidden.split(";")
        randomized = []
        
        if self.transform == 'Selected':
            objects = currentScene.objects
            for object in objects:
                if object.select:
                    if object.slideshow.name != "None":
                        if not object.slideshow.locktransform:
                            object.slideshow.transform = currentSlide.slideshow.transform

        else:
            #Iterate through transforms to get a list of non-disabled ones
            for transform in transforms:
                if transform['name'] not in hidden:
                    randomized.append(transform)
            lasttransform = {"name":""}
            
            #Iterate through all slides
            for slide in slides:
                if not slide.slideshow.locktransform:
                    #Only modify slides without a locked transform
                    if self.transform == 'Random':
                        #Apply a randomized transform to all slides
                        if len(randomized) > 0:
                            #Only can be applied if at least one transform is enabled
                            if len(randomized) > 1:
                                #More than one transform is enabled, apply a random one while making sure it is not the same as the last applied transform
                                lessrandomized = [x for x in randomized if x['name'] != lasttransform['name']]
                                newtransform = lessrandomized[random.randint(0, (len(lessrandomized)-1))]
                                slide.slideshow.transform = newtransform['name']
                                lasttransform = newtransform
                            else:
                                #Only one transform is enabled, just apply it
                                slide.slideshow.transform = randomized[0]['name']
                    else:
                        #Apply the named transform to the slide
                        slide.slideshow.transform = currentSlide.slideshow.transform
                        
        return{'FINISHED'}


class SlideshowApplySlideLength(bpy.types.Operator):
    #This operator will apply a slide length to all slides that do not have a locked length
    bl_idname = 'slideshow.apply_slide_length'
    bl_label = 'Apply To All'
    bl_description = 'Applies current slide length to all slides'
    
    mode = bpy.props.StringProperty()
    
    def execute(self, context):
        currentScene = bpy.context.scene
        currentSlide = currentScene.objects.active
        if self.mode == 'Selected':
            objects = currentScene.objects
            for object in objects:
                if object.select:
                    if object.slideshow.name != "None":
                        if not object.slideshow.locklength:
                            object.slideshow.length = currentSlide.slideshow.length

        else:
            slides = list_slides(currentScene)
            
            for slide in slides:
                if not slide.slideshow.locklength:
                    slide.slideshow.length = currentSlide.slideshow.length
        return{'FINISHED'}


class SlideshowUpdateOrder(bpy.types.Operator):
    #This operator just runs the 'update_order' function
    bl_idname = 'slideshow.update_order'
    bl_label = 'Update Slide Order'
    
    mode = bpy.props.StringProperty()
    
    def execute(self, context):
        update_order(self.mode)
        return{'FINISHED'}


class SlideshowDeleteSlide(bpy.types.Operator):
    #This operator will remove all selected slides from the generator scene
    bl_idname = 'slideshow.delete_slide'
    bl_label = 'Delete Selected Slide(s)'
    bl_description = 'Deletes selected slides and rearranges the list'
    
    def execute(self, context):
        currentScene = bpy.context.scene
        selected_objects = bpy.context.selected_objects
        
        #Iterate through selected objects
        for selected in selected_objects:
            if len(selected_objects) == 1 and selected.slideshow.name == "None" and selected.parent:
                #Only one object is selected, and the selected object is one of the sub objects of a slide - the user probably wants to delete that slide, so change the selection to that
                selected = selected.parent
            
            if selected.slideshow.name != "None":
                #The selected object is a slide, delete it and all its sub-objects
                bpy.ops.object.select_all(action='DESELECT')
                group = selected.users_group[0]
                for object in group.objects:
                    object.select = True
                bpy.ops.object.delete()
                update_order()
        return{'FINISHED'}


class SlideshowAddExtraTexture(bpy.types.Operator):
    #This operator will add a texture preset to the extra textures
    bl_idname = 'slideshow.add_extra_texture'
    bl_label = '+'
    bl_description = 'Adds a texture preset to the extra textures'
    
    texture = bpy.props.StringProperty()
    
    def execute(self, context):
        texturename = os.path.split(self.texture)[1]
        if bpy.context.scene.slideshowExtraTexturePresets.find(texturename) == -1:
            newtexture = bpy.context.scene.slideshowExtraTexturePresets.add()
            newtexture.name = texturename
            newtexture.path = self.texture
            self.report({'INFO'}, "Added Preset: "+texturename)
        else:
            self.report({'INFO'}, "Preset Already Exists: "+texturename)
        return{'FINISHED'}


class SlideshowRemoveExtraTexture(bpy.types.Operator):
    #This operator will attempt to find and remove a texture preset from the list
    bl_idname = 'slideshow.remove_extra_texture'
    bl_label = '-'
    bl_description = 'Removes a texture preset from the extra textures'
    
    texture = bpy.props.StringProperty()
    
    def execute(self, context):
        remove = os.path.split(self.texture)[1]
        index = bpy.context.scene.slideshowExtraTexturePresets.find(remove)
        if index >= 0:
            bpy.context.scene.slideshowExtraTexturePresets.remove(index)
            self.report({'INFO'}, "Removed Preset: "+remove)
        else:
            self.report({'INFO'}, "Could Not Find Preset: "+remove)
        return{'FINISHED'}


class SlideshowExtraTextureMenu(bpy.types.Menu):
    #This is a menu to list extra texture presets
    bl_idname = 'slideshow.extra_texture_menu'
    bl_label = 'List of saved extra textures'
    
    def draw(self, context):
        layout = self.layout
        layout.operator("slideshow.change_extra_texture", text="None").texture = "None"
        for texture in bpy.context.scene.slideshowExtraTexturePresets:
            layout.operator("slideshow.change_extra_texture", text=texture.name).texture = texture.path


class SlideshowChangeExtraTexture(bpy.types.Operator):
    #Set the extra texture for the currently active slide
    bl_idname = 'slideshow.change_extra_texture'
    bl_label = 'Change Extra Texture'
    
    texture = bpy.props.StringProperty()
    
    def execute(self, context):
        currentScene = bpy.context.scene
        if currentScene.name == 'Slideshow Generator':
            currentImage = currentScene.objects.active
            currentImage.slideshow.extratexture = self.texture
        else:
            currentScene.slideshowExtraTexture = self.texture
        return{'FINISHED'}


class SlideshowCreate(bpy.types.Operator):
    #This operator creates a slideshow scene and all the image scenes from the generator scene
    bl_idname = 'slideshow.create'
    bl_label = 'Create Slideshow'
    bl_description = 'Turns the Slideshow Generator scene into a full slideshow'
    
    def execute(self, context):
        generatorScene = bpy.data.scenes['Slideshow Generator']
        
        #Set up the Slideshow scene
        if 'Slideshow' in bpy.data.scenes:
            bpy.data.scenes.remove(bpy.data.scenes['Slideshow'])
        slideshowScene = create_scene(generatorScene, 'Slideshow')
        imageSceneStart = 1
        
        #Create a copy of the currently active and selected objects so they can be re-set later
        active = bpy.context.scene.objects.active
        selected = []
        for object in bpy.context.scene.objects:
            if object.select:
                selected.append(object)

        bpy.context.screen.scene = slideshowScene

        #Iterate through image list, create scene for each image, and import into VSE
        images = list_slides(generatorScene)
        images.sort(key=lambda x: x.slideshow.index)
        previousImageClip = None
        for i, imagePlane in enumerate(images):
            previousImageClip = create_slideshow_slide(imagePlane, i, generatorScene, slideshowScene, imageSceneStart, images, previousImageClip)
            imageSceneStart = previousImageClip.frame_final_end - generatorScene.slideshowCrossfadeLength
        self.report({'INFO'}, "Slideshow created")
        
        #Set slideshow length
        slideshowScene.frame_end = imageSceneStart + generatorScene.slideshowCrossfadeLength
        slideshowScene.sync_mode = 'AUDIO_SYNC'
        
        #Add slideshow audio if enabled
        if generatorScene.slideshowAudioEnabled:
            filename = os.path.realpath(bpy.path.abspath(generatorScene.slideshowAudioTrack))
            if os.path.exists(filename):
                extension = os.path.splitext(generatorScene.slideshowAudioTrack)[1].lower()
                if extension in bpy.path.extensions_audio:
                    #audio is enabled, the file exists, and is an audio format blender recognizes
                    audio_frame_end = 0
                    
                    #This loop will repeat adding the audio track until the end of the slideshow is reached
                    i = 0
                    while audio_frame_end < slideshowScene.frame_end:
                        if audio_frame_end == 0:
                            #Set the starting point for the first loop
                            frame_start = 1
                        else:
                            #Set the starting point for a repeating audio loop
                            frame_start = audio_frame_end+1-generatorScene.slideshowAudioLoopFade
                        
                        audioSequence = slideshowScene.sequence_editor.sequences.new_sound(name='Audio', filepath=filename, channel=6+(i%2), frame_start=frame_start)
                        
                        if audioSequence.frame_duration > 0:
                            #The audio sequence was added correctly, set the ending point
                            audio_frame_end = audioSequence.frame_final_end
                        else:
                            #Something went wrong, abort adding audio sequences
                            audio_frame_end = slideshowScene.frame_end
                        i += 1

                    #Add a fadeout to the last audio sequence
                    audioSequence.frame_final_end = slideshowScene.frame_end
                    slideshowScene.frame_current = slideshowScene.frame_end - generatorScene.slideshowAudioFadeLength
                    audioSequence.keyframe_insert(data_path='volume')
                    slideshowScene.frame_current = slideshowScene.frame_end
                    audioSequence.volume = 0
                    audioSequence.keyframe_insert(data_path='volume')
        
        #Attempt to switch to the video editor screen layout
        try:
            bpy.context.window.screen = bpy.data.screens['Video Editing']
            bpy.data.screens['Video Editing'].scene = slideshowScene
        except KeyError:
            pass

        slideshowScene.render.sequencer_gl_preview = 'MATERIAL'
        try:
            slideshowScene.render.use_sequencer_gl_preview = True
        except:
            pass
        
        #Return the selected and active objects to the way they were before the slideshow generation
        for object in bpy.context.scene.objects:
            if object in selected:
                object.select = True
            else:
                object.select = False
        bpy.context.scene.objects.active = active
        
        return{'FINISHED'}


class SlideshowGenerator(bpy.types.Operator):
    #This operator will import images, and create the slideshow generator scene and everything in it
    bl_idname = 'slideshow.generator'
    bl_label = 'Create Slideshow Generator'
    bl_description = 'Imports images and creates a scene for setting up the slideshow'
    
    def execute(self, context):
        slideshowSlideLength = bpy.context.scene.slideshowSlideLength
        slideshowImageDirectory = bpy.context.scene.slideshowImageDirectory
        imageTypes = list(bpy.path.extensions_image)
        videoTypes = list(bpy.path.extensions_movie)
        
        #get list of images in slideshowImageDirectory
        imageList = []
        videoList = []
        for type in imageTypes:
            #Search directory for files matching all image types, and add them to the list
            imageList.extend(glob.glob(bpy.path.abspath(slideshowImageDirectory)+'*'+type))
        for type in videoTypes:
            videoList.extend(glob.glob(bpy.path.abspath(slideshowImageDirectory)+'*'+type))
        totalImages = len(imageList)+len(videoList)
        
        #check if slideshow generator scene exists, warn and cancel if it does
        if (bpy.data.scenes.find('Slideshow Generator') != -1):
            self.report({'WARNING'}, 'Slideshow Generator Scene Already Exists')
            return{'CANCELLED'}
        
        self.report({'INFO'}, 'Importing '+str(totalImages)+' images')
        
        #create scene, switch to it, and set up properties
        oldscene = bpy.context.scene
        slideshowGeneratorScene = create_scene(oldscene, 'Slideshow Generator')
        slideshowGeneratorScene.slideshowCrossfadeLength = 10
        slideshowGeneratorScene.slideshowSlideLength = slideshowSlideLength
        slideshowGeneratorScene.slideshowTransformsHidden = oldscene.slideshowTransformsHidden
        slideshowGeneratorScene.slideshowExtrasHidden = oldscene.slideshowExtrasHidden
        for extraTexturePreset in oldscene.slideshowExtraTexturePresets:
            newPreset = slideshowGeneratorScene.slideshowExtraTexturePresets.add()
            newPreset.name = extraTexturePreset.name
            newPreset.path = extraTexturePreset.path
        
        #Attempt to switch to the correct viewport view and settings
        for area in bpy.context.screen.areas:
            if (area.type == 'VIEW_3D'):
                for space in area.spaces:
                    if (space.type == 'VIEW_3D'):
                        space.viewport_shade = 'TEXTURED'
                        space.region_3d.view_rotation = (1.0, 0, 0, 0)
                        space.region_3d.view_perspective = 'ORTHO'
        
        #Add header instruction text
        instructionsData = bpy.data.curves.new(name='Instructions', type='FONT')
        instructions = bpy.data.objects.new(name='Instructions', object_data=instructionsData)
        slideshowGeneratorScene.objects.link(instructions)
        instructions.location = ((-1, 1.25, 0.0))
        instructions.scale = ((.15, .15, .15))
        instructions.data.body = "Select an image and see the Scene tab in the properties area for details.\nDrag an image to rearrange it in the timeline.\nThe center cross on each image represents the focal point for transformations.\nThe box surrounding each image represents the viewable area for the cameara.\nMove, scale, and rotate this to change the viewable area."

        #Iterate through the list and import the images
        imageNumber = 1
        lastImage = None
        for imageFile in imageList:
            image = load_image(imageFile)
            lastImage = import_slideshow_image(image, imageNumber, slideshowSlideLength, slideshowGeneratorScene, video=False, lastImage=lastImage)
            imageNumber += 1
        
        for videoFile in videoList:
            image = load_image(videoFile)
            lastImage = import_slideshow_image(image, imageNumber, slideshowSlideLength, slideshowGeneratorScene, video=True, lastImage=lastImage)
            imageNumber += 1
        
        bpy.context.scene.cursor_location = (0, 0, 0)
        print("Now displaying images, this may take a while...")
        
        #For some reason, blender refuses to update the scene if it is currently active, so work around needed...
        bpy.context.screen.scene = oldscene
        slideshowGeneratorScene.update()
        bpy.context.screen.scene = slideshowGeneratorScene
        
        return{'FINISHED'}


def register():
    bpy.utils.register_module(__name__)
    
    bpy.types.Scene.slideshowTransformsHidden = bpy.props.StringProperty(
        name = "Hidden Transforms",
        default = "",
        description = "List of transforms to not use in randomize operations")
    bpy.types.Scene.slideshowExtrasHidden = bpy.props.StringProperty(
        name = "Hidden Extras",
        default = "Text Normal Bottom;Text Normal Top;Parallax Frame Overlay Landscape;Parallax Frame Overlay Portrait;Video Background;Video Foreground;Compositor Glare;Overlay Curves Left;Overlay Curves Right",
        description = "List of extras to not use in randomize operations")

    bpy.types.Scene.slideshowImageDirectory = bpy.props.StringProperty(
        name = "Image Directory",
        default = '/Images/',
        description = "Location of images used in slideshow",
        subtype='DIR_PATH')
    
    bpy.types.Scene.slideshowSlideLength = bpy.props.FloatProperty(
        name = "Slide Length (Seconds)",
        default = 4,
        min = 1,
        max = 20,
        description = "Slide Scene Length In Seconds")
    
    bpy.types.Scene.slideshowCrossfadeLength = bpy.props.IntProperty(
        name = "Crossfade Length (Frames)",
        default = 10,
        min = 0,
        max = 120)

    bpy.types.Scene.slideshowExtraTexture = bpy.props.StringProperty(
        name = "Extra Texture",
        default = "None")
    bpy.types.Scene.slideshowExtraTexturePresets = bpy.props.CollectionProperty(
        type=SlideshowExtraTexturePreset)

    bpy.types.Scene.slideshowAudioEnabled = bpy.props.BoolProperty(
        default = False)
    bpy.types.Scene.slideshowAudioTrack = bpy.props.StringProperty(
        name = "Audio Track",
        default = "None")
    bpy.types.Scene.slideshowAudioFadeLength = bpy.props.IntProperty(
        name = "Fade Out",
        description = "Length of audio fade out in frames",
        default = 30,
        min = 0,
        max = 600)
    bpy.types.Scene.slideshowAudioLoopFade = bpy.props.IntProperty(
        name = "Loop Overlap",
        description = "Length of audio overlap when track is looped in frames",
        default = 60,
        min = 0,
        max = 600)
        
    bpy.types.Object.slideshow = bpy.props.PointerProperty(type=SlideshowImage)
    
    handlers = bpy.app.handlers.scene_update_post
    for handler in handlers:
        if (" slideshow_autoupdate " in str(handler)):
            handlers.remove(handler)
    handlers.append(slideshow_autoupdate)


def unregister():
    bpy.utils.unregister_module(__name__)
    del bpy.types.Scene.slideshowImageDirectory
    del bpy.types.Scene.slideshowSlideLength
    handlers = bpy.app.handlers.scene_update_post
    for handler in handlers:
        if (" slideshow_autoupdate " in str(handler)):
            handlers.remove(handler)


if __name__ == "__main__":
    register()